/*******************************************************************************
* FILE NAME: auto_routines.c
*
* DESCRIPTION:
*  Sample autonomous code that maintains a set distance from any wall or object
*  A user adjustable potentiometer tells us how far we should stay away
*  If the robot is right where it should be we just sit still
*  If we're too far away we go forward at a speed proportional to how far off we are
*  If we're too close we backup
*
*  For example, if we're only a few inches away from where we should be, the robot will
*  inch forward, but if we're 2 feet away we'll accelerate to get there sooner and
*  slow down as we approach our target distance.*
*******************************************************************************/

#include "ifi_aliases.h"
#include "ifi_default.h"
#include "ifi_utilities.h"
#include "user_routines.h"
#include "stdio.h"
#include "Ultrasonic_sensor.h"

#define KP 50/10  // Basic proportional adjustment (50/10 works fine)
#define MIN_SPEED 20 //Any slower and the robot wouldn't be moving
#define MAX_SPEED 127
#define DEFAULT_DISTANCE 20 // About a foot or so away
#define NEUTRAL 127  //Motors are not moving 

/*******************************************************************************
* FILE NAME: Auto_Distance
*
* DESCRIPTION:
*   Simply demonstrates maintaining a fixed distance from a wall or other object
*******************************************************************************/
void Auto_Distance (void)
{
  unsigned int pot;
  int proportion;
  int temp;


// Maintain a constant distance from a wall or object
  pot  = Get_Analog_Value(POT_INPUT)/10; // How far should we stay away?

  // A little fault detection here. A pot failure could send us off to smash into a wall at full speed.
  if (pot == 101) pot = DEFAULT_DISTANCE; //101 is out of sonic range and could be a disconnected pot

  //******** This distance is just right ********
  if ( (fltHiResCounter >= pot) && (fltHiResCounter <= (pot+DISTANCE_TOLERANCE)) )
  {
    pwm01 = NEUTRAL; // Sit still
    pwm02 = NEUTRAL; // Be still my motors
  }

  //******** This distance is too far ********
  else if (fltHiResCounter > pot)
  {
    // Make our motor reaction proportional to how far we are from where we want to be
    // If we're close then go slowly so we don't overshoot
    // If we're farther away then we can go full speed to get where we want to be sooner
    proportion = (int)(fltHiResCounter - pot) * KP;
    if (proportion > MAX_SPEED) proportion = MAX_SPEED;
    if (proportion < MIN_SPEED) proportion = MIN_SPEED;

    pwm01 = NEUTRAL + proportion; // Backward proportional to error (254)
    pwm02 = NEUTRAL - proportion; // Backward proportional to error (0)
  }

  //******** This distance is too close ********
  else if (fltHiResCounter < (pot+DISTANCE_TOLERANCE))
  {
    proportion = (int)(pot - fltHiResCounter) * KP;
    if (proportion > MAX_SPEED) proportion = MAX_SPEED;
    if (proportion < MIN_SPEED) proportion = MIN_SPEED;

    pwm01 = NEUTRAL - proportion; // Forward proportional to error (0)
    pwm02 = NEUTRAL + proportion; // Forward proportional to error (254)
  }

  // Match the motor power on both sides
  temp = (int)pwm02 - 127;
  if (pwm02 < 127) pwm02 -= FORWARDBALANCE*temp/127;
  temp = (int)pwm01 - 127;
  if (pwm01 < 127) pwm01 -= BACKWARDBALANCE*temp/127;
}
